CREATE OR REPLACE PACKAGE JSON AS
/******************************************************************************
   NAME:       JSON
   PURPOSE:

   REVISIONS:
   Ver        Date        Author           Description
   ---------  ----------  ---------------  ------------------------------------
   1.0        03/07/2007             1. Created this package.
******************************************************************************/
--------------------------------------------------------------------------------
-- Global Types and records
--------------------------------------------------------------------------------
-- type for JSON Array
type JSONArray is table of varchar2(2000) index by binary_integer;

-- type for all Name/Value Couples in JSON
type gr_JSONnvCouple is record (name varchar2(255), value varchar2(2000));

-- Type for the final JSON generated string
type JSONItem is record ( type varchar2(100),  -- OPENBRACKET, OPENHOOK, CLOSEBRACKET, CLOSEHOOK, 
	 		  	 		  	   				   -- SEPARATION, AFFECTATION, ATTRNAME, ATTRDATA, ARRAYDATA
											   -- INDENTATION
	 		  	 		  item varchar2(2000), -- the attribute name or value.
						  formated boolean default false); -- true if "item" has been already formatted.				  
type JSONStructObj is table of JSONItem index by binary_integer;

--------------------------------------------------------------------------------
-- Global variables and constants
--------------------------------------------------------------------------------
-- Output redirection. Possibles values are 
-- 		  -> 'WEB' (via Oracle htp/htf toolkit) Default
--		  -> 'CUSTO' (Your custom output)
g_output_type constant varchar2(10) := 'WEB';

-- 
g_objOpenBracket     varchar2(2) := '{ ';
g_objcloseBracket 	 varchar2(2) := ' }';
g_arrayOpenBracket 	 varchar2(2) := '[ ';
g_arraycloseBracket  varchar2(2) := ' ]';
g_stringDelimiter 	 varchar2(1) := '''';
g_Affectation 		 varchar2(3) := ' : ';
g_separation 	 	 varchar2(3) := ', ';
g_CR				 varchar2(1) := Chr(10); -- used to indent the JSON object correctly
g_spc				 varchar2(2) := '  ';	 -- used to indent the JSON object correctly

g_indent varchar2(2000) := null;  -- count the recursive imbrications for object 
		  				 	   	  -- +2 spaces when calling openObj 
								  -- -2 spaces when calling closeObj

g_package_version constant varchar2(100) := '1.0 Stable';
--------------------------------------------------------------------------------
-- Public proc. and  funct. signatures
--------------------------------------------------------------------------------
procedure newJSONObj(p_obj in out nocopy JSONStructObj, p_doindetation boolean default true);
procedure closeJSONObj(p_obj in out nocopy JSONStructObj);
function addAttr(p_obj JSONStructObj, n varchar2, v varchar2, p_formated boolean default false) return JSONStructObj;
function addAttr(p_obj JSONStructObj, n varchar2, pbool boolean, p_formated boolean default false) return JSONStructObj;
function addAttr(p_obj JSONStructObj, n varchar2, p_objValue JSONStructObj) return JSONStructObj;
function addArray(p_tab JSONArray, p_format boolean default false) return JSONStructObj;
function addArray(p_obj JSONStructObj, p_table JSONArray, p_formated boolean default false) return JSONStructObj;
function JSON2String(p_obj in out nocopy JSONStructObj, p_only_an_array boolean default false) return varchar2;
function String2JSON(p_str varchar2, pStrDelimiter varchar2 default g_stringDelimiter) return JSONStructObj;
procedure HTMLdumpJSONObj(p_obj in out nocopy JSONStructObj);
function getAttrValue(p_obj JSONStructObj, pname varchar2, pdecode boolean default true) return varchar2;
function setAttrSimpleValue(p_obj JSONStructObj, pname varchar2, pvalue varchar2, pformated boolean default false) return JSONStructObj;
function setAttrSimpleValue(p_obj JSONStructObj, pname varchar2, pbool boolean, pformated boolean default false) return JSONStructObj;
function validateJSONObj(p_obj in out nocopy JSONStructObj, pvalidate boolean default false) return pls_integer;
procedure print(p_str varchar2);
function getVersion return varchar2;
procedure test;

END JSON;
/
